/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <zephyr.h>
#include <stdio.h>
#include <uart.h>
#include <string.h>
#include <stdlib.h>
#include <main.h>

#include <adc.h>
#include <spi.h>
#include <i2c.h>
#include <drivers/gpio.h>

#include <net/mqtt.h>
#include <net/socket.h>
#include <lte_lc.h>
#if defined(CONFIG_LWM2M_CARRIER)
#include <lwm2m_carrier.h>
#endif

#include <MT25QL128ABA.h>
#include <afe4900.h>
#include <adp536x.h>
#include <lis2dh12.h>

#include <hal/nrf_saadc.h>
#include <stdarg.h>
//#include "SEGGER_RTT.h"

#define TIEMR_INTERVAL_SEC 1
struct k_timer my_timer;

#define NO_MQTT   
//#undef NO_MQTT


#define GPIO_INT_PIN    17 
static struct gpio_callback gpio_cb;


#define RLED  0
#define GLED  2
#define BLED  1
#define POW_ENA 19
#define OSC_ENA 20

#define RED   0
#define GREEN 2
#define BLUE  1


#define ON    true
#define OFF   false


 


MQTT_SM_STATUS       _MQTT_SM_STATUS;
MQTT_CONNECT_STATUS  _MQTT_CONNECT_STATUS;
MQTT_SLEEP_STATUS    _MQTT_SLEEP_STATUS;
MQTT_ACCEL_STATUS    _MQTT_ACCEL_STATUS;
CURRENT_DATE_TIME    _CURRENT_DATE_TIME;
 
AFE4900_SM           _AFE4900_SM;
AFE4900_OPERATE      _AFE4900_OPERATE;
ESP008_OPERATE       _ESP008_OPERATE;
AFE4900_LOW_LEVEL_SM _AFE4900_LOW_LEVEL_SM;
AFE4900_RECIPE       _AFE4900_RECIPE;
SLEEP_SM             _SLEEP_SM;
LED_SM               _LED_SM;
AFE4900_AQUIRE_MODE  _AFE4900_AQUIRE_MODE;
MT25QI128_MEMORY_SM  _MT25QI128_MEMORY_SM;
ESP008_OPERATE_STATUS _ESP008_OPERATE_STATUS;
ESP008_OPERATE       _ESP008_OPERATE;
MT25QI128_MEMORY     _MT25QI128_MEMORY; 

 
struct device * spi_dev1;
struct device * spi_dev3;
struct device * i2c_dev2;
struct device * dev_io;
struct device * uart_dev1;
struct device * adc_dev; 


#define ADC_DEVICE_NAME DT_ADC_0_NAME
#define ADC_RESOLUTION 10
#define ADC_GAIN ADC_GAIN_1_6
#define ADC_REFERENCE ADC_REF_INTERNAL
#define ADC_ACQUISITION_TIME ADC_ACQ_TIME(ADC_ACQ_TIME_MICROSECONDS, 10)
#define ADC_1ST_CHANNEL_ID 0
#define ADC_1ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN0
#define ADC_2ND_CHANNEL_ID 2
#define ADC_2ND_CHANNEL_INPUT NRF_SAADC_INPUT_AIN2

static const struct adc_channel_cfg m_1st_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_1ST_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_1ST_CHANNEL_INPUT,
#endif
};



#define BUFFER_SIZE 1
static u16_t m_sample_buffer[BUFFER_SIZE];


/* Buffers for MQTT client. */
static u8_t rx_buffer[CONFIG_MQTT_MESSAGE_BUFFER_SIZE];
static u8_t tx_buffer[CONFIG_MQTT_MESSAGE_BUFFER_SIZE];
static u8_t payload_buf[CONFIG_MQTT_PAYLOAD_BUFFER_SIZE];

/* The mqtt client struct */
static struct mqtt_client client;

/* MQTT Broker details. */
static struct sockaddr_storage broker;

/* Connected flag */
static bool connected;

/* File descriptor */
static struct pollfd fds;

static int data_publish(struct mqtt_client *c, enum mqtt_qos qos,
	u8_t *data, size_t len);
static void data_check(u8_t *data, size_t len);
static void data_print(u8_t *prefix, u8_t *data, size_t len);

bool new_command = false;


 static unsigned int connect_off = 0;
 static unsigned int connect_on = 0;


unsigned int gtimer;
unsigned int future_time;

unsigned char afe4900_result[10];
unsigned char flash_buffer[2][3200];
unsigned int flash_page_counter = 0;
unsigned int flash_section_counter = 0;
unsigned char afe4900_int_counter = 0;
unsigned char afe4900_mem_chunk = 6;
unsigned char lis2dh12_mem_chunk = 6;
unsigned char adc_mem_chunk = 2;
unsigned char etrog_mem_chunk = 8;
unsigned char flash_bank = 0;
unsigned char flash_chunk_counter = 0;
unsigned char flash_block_switch = false;




unsigned short adc_result;


u8_t   spi_tx_buffer[260];
u8_t   spi_rx_buffer[260]; 

struct spi_buf tx_buf;
struct spi_buf_set tx;
struct spi_buf rx_buf;
struct spi_buf_set rx;

//struct spi_cs_control lis2dh12_spi_cs = {
//	.gpio_pin = 4 ,  /*CS_PIN_NUM(CONFIG_ST25R3911B_SPI_PORT), */
//	.delay = 1, /* 8T_NCS_SCLK */ 
//};
//
//struct spi_config spi_cfg_lis = {
//	.operation = SPI_OP_MODE_MASTER | SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
//		      SPI_MODE_CPOL | SPI_MODE_CPHA ,
//	.frequency = 7000000,
//	.slave = 0,
//        .cs = &lis2dh12_spi_cs,     
//};


struct spi_cs_control afe4900_spi_cs = {
	.gpio_pin = 26 ,  /*CS_PIN_NUM(CONFIG_ST25R3911B_SPI_PORT), */
	.delay = 0, /* 8T_NCS_SCLK */ 
};

struct spi_config spi_cfg_afe4900 = {
	.operation =SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
    //    .operation = SPI_OP_MODE_MASTER | SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
		      SPI_MODE_CPOL | SPI_MODE_CPHA ,
	.frequency = 7000000,
	.slave = 0,
       .cs = &afe4900_spi_cs,     
};


struct spi_cs_control flash_spi_cs = {
	.gpio_pin = 15 ,  /*CS_PIN_NUM(CONFIG_ST25R3911B_SPI_PORT), */
	.delay = 0, /* 8T_NCS_SCLK */ 
};

struct spi_config spi_cfg_flash = {  // ****** NOTICE ME! ********
	.operation =   SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
		      SPI_MODE_CPOL | SPI_MODE_CPHA ,
	.frequency = 7000000,
	.slave = 0,
        .cs = &flash_spi_cs,     
};

//
//
//
//struct spi_config spi_cfg_flash = {
//	.operation = SPI_OP_MODE_MASTER | SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
//		      SPI_MODE_CPOL | SPI_MODE_CPHA ,
//	.frequency = 7000000,
//	.slave = 0,
//        .cs = &flash_spi_cs,        
//};


#if defined(CONFIG_BSD_LIBRARY)

 

/**@brief Recoverable BSD library error. */
void bsd_recoverable_error_handler(uint32_t err)
{
	printk("bsdlib recoverable error: %u\n", (unsigned int)err);
}

#endif /* defined(CONFIG_BSD_LIBRARY) */

#if defined(CONFIG_LWM2M_CARRIER)
K_SEM_DEFINE(carrier_registered, 0, 1);

void lwm2m_carrier_event_handler(const lwm2m_carrier_event_t *event)
{
	switch (event->type) {
	case LWM2M_CARRIER_EVENT_BSDLIB_INIT:
		printk("LWM2M_CARRIER_EVENT_BSDLIB_INIT\n");
		break;
	case LWM2M_CARRIER_EVENT_CONNECT:
		printk("LWM2M_CARRIER_EVENT_CONNECT\n");
		break;
	case LWM2M_CARRIER_EVENT_DISCONNECT:
		printk("LWM2M_CARRIER_EVENT_DISCONNECT\n");
		break;
	case LWM2M_CARRIER_EVENT_READY:
		printk("LWM2M_CARRIER_EVENT_READY\n");
		k_sem_give(&carrier_registered);
		break;
	case LWM2M_CARRIER_EVENT_FOTA_START:
		printk("LWM2M_CARRIER_EVENT_FOTA_START\n");
		break;
	case LWM2M_CARRIER_EVENT_REBOOT:
		printk("LWM2M_CARRIER_EVENT_REBOOT\n");
		break;
	}
}
#endif /* defined(CONFIG_LWM2M_CARRIER) */

/**@brief Function to print strings without null-termination
 */
static void data_print(u8_t *prefix, u8_t *data, size_t len)
{
	char buf[len + 1];

	memcpy(buf, data, len);
	buf[len] = 0;
	printk("%s%s\n", prefix, buf);
}


void StopActivity(void)
{
    // Init _AFE4900_OPERATE Structure.
    _AFE4900_OPERATE.afe4900_operate_status = idle;
    _AFE4900_SM.afe4900_test_type = none;

    // ESP008 Operate Structure.
    _ESP008_OPERATE.esp008_operate_status = esp_idle;

    // _AFE4900_RECIPE Structure.
    _AFE4900_RECIPE.test_preset = afe4900_test_idle; 
    _AFE4900_RECIPE.test_type = none;
    _AFE4900_RECIPE.recipe_duration = 12;
     _AFE4900_RECIPE.recipe_duration = 1;
    _AFE4900_RECIPE.recipes_num = 1; 
    _AFE4900_RECIPE.start_time = 0;
    _AFE4900_RECIPE.stop_time = 1;
    _AFE4900_RECIPE.recipe_interval = 1;
    _AFE4900_RECIPE.save = true;
    _AFE4900_RECIPE.wire = true;

    // AFE4900 lOW lEVEL sm.
    _AFE4900_LOW_LEVEL_SM.m_state = afe4900_low_level_idle; 

    // Start Aquire mode
   _AFE4900_AQUIRE_MODE.aquire_mode = afe4900_aquire_idle;
   _AFE4900_AQUIRE_MODE.spi3_state = spi3_idle;
}

int io_init(void)
{
    const char* const ioName = "GPIO_0";

    dev_io = device_get_binding(ioName);
  
    printk("Main \r\n"); 
    dev_io =  device_get_binding(ioName);
    if (dev_io == NULL) {
            printk("Could not get %s device\n", ioName);
//            gpio_pin_configure(dev_io, RLED, GPIO_DIR_OUT); 
//             SetLed(RED,ON);
//            while(1) {};
            return;

    }
    else
    {
      printk("GPIO_0 found \r\n"); 
    }

    // Set Led IO.s
    gpio_pin_configure(dev_io, RLED, GPIO_DIR_OUT); 
    gpio_pin_configure(dev_io, GLED, GPIO_DIR_OUT); 
    gpio_pin_configure(dev_io, BLED, GPIO_DIR_OUT); 
    // Set POW_ENA IO.
    gpio_pin_configure(dev_io, POW_ENA , GPIO_DIR_OUT); 
    // Set OSC_ENA IO.
    gpio_pin_configure(dev_io, OSC_ENA , GPIO_DIR_OUT); 
    // cs_afe4900

 //   gpio_pin_configure(dev_io, 26 , GPIO_DIR_OUT); 
 //   gpio_pin_write(dev_io, 26, 1);
  
    afe4900_spi_cs.gpio_dev = device_get_binding("GPIO_0");
    afe4900_spi_cs.gpio_pin = 26;
    afe4900_spi_cs.delay = 1;

    flash_spi_cs.gpio_dev = device_get_binding("GPIO_0");
    flash_spi_cs.gpio_pin = 15;
    flash_spi_cs.delay = 1;



    // afe4900 Reset.
    gpio_pin_configure(dev_io, 16 , GPIO_DIR_OUT); 
    gpio_pin_write(dev_io, 16, 0);    
    
    // Lis2dh12 CS.
    gpio_pin_configure(dev_io, 4 , GPIO_DIR_OUT); 
    gpio_pin_write(dev_io, 4, 1);                       
    // cs_flash
    gpio_pin_configure(dev_io, 15 , GPIO_DIR_OUT); 
    gpio_pin_write(dev_io, 15, 1);  
    // afe4900 CS.
 //   gpio_pin_configure(dev_io, 26 , GPIO_DIR_OUT); 
 //   gpio_pin_write(dev_io, 26, 1);

    // reset spi3 ip
//    gpio_pin_configure(dev_io, 22 , GPIO_DIR_OUT); 
//    gpio_pin_write(dev_io, 22, 1);  
//    gpio_pin_configure(dev_io, 23 , GPIO_DIR_OUT); 
//    gpio_pin_write(dev_io, 23, 0);
//    gpio_pin_configure(dev_io, 25 , GPIO_DIR_OUT); 
//    gpio_pin_write(dev_io, 25, 0);  
//    gpio_pin_configure(dev_io, 30 , GPIO_DIR_OUT); 
//    gpio_pin_write(dev_io, 30, 0);  
//    gpio_pin_configure(dev_io, 31 , GPIO_DIR_OUT); 
//    gpio_pin_write(dev_io, 31, 0);  
//    gpio_pin_configure(dev_io, 9 , GPIO_DIR_OUT); 
//    gpio_pin_write(dev_io, 9, 1);  

}

int adc_init(void)
{
     int err = true;

      adc_dev = device_get_binding("ADC_0");
 //     if (!adc_dev)
      if (adc_dev == NULL)
      {
         printk("efi device_get_binding ADC_0 failed\n");
         err = false;
      }
      else
      {
         err = adc_channel_setup(adc_dev, &m_1st_channel_cfg);
         if (err)
         {
              printk("Error in adc setup: %d\n", err);
              err = false;       
         }
         else
         {
           printk("adc setup is OK \n" );
           err = true;
         }
      }
     return err;
} 

int spi_init1(void)
{
        const char* const spiName1 = "SPI_1";
         
	spi_dev1 = device_get_binding(spiName1);

	if (spi_dev1 == NULL) {
		printk("Could not get %s device\r\n", spiName1);
		return 0;
        } 
        else
        {
                printk("device %s is configured\r\n", spiName1);
		return 1;
        }
}

//int uart_init1(void)
//{
//         const char* const uartName1 = "UART_1";
//
//	uart_dev1 = device_get_binding(uartName1);
//
//	if (uart_dev1 == NULL) {
//		printk("Could not get %s device\r\n", uartName1);
//		return 0;
//        } 
//        else
//        {
//                printk("device %s is configured\r\n", uartName1);
//		return 1;
//        }
//}



int spi_init3(void)
{
         const char* const spiName3 = "SPI_3";

	spi_dev3 = device_get_binding(spiName3);

	if (spi_dev3 == NULL) {
		printk("Could not get %s device\r\n", spiName3);
		return 0;
        } 
        else
        {
                printk("device %s is configured\r\n", spiName3);
		return 1;
        }
}

uint8_t init_i2c2()
 {
        const char* const i2cName2 = "I2C_2";

        i2c_dev2 = device_get_binding(i2cName2);

        if(i2c_dev2 == NULL)
        {
           printk("Could not get %s device\r\n", i2cName2);
           return 0;
        }
        else
        {
           i2c_configure(i2c_dev2,I2C_SPEED_SET(I2C_SPEED_STANDARD));
          printk("device %s is configured\r\n", i2cName2);
           return 1;
        }    
  }




static const char *known_commands[] = {"IDLE","STOP","TIME","PRESET","INTERVAL","CONT","SLEEP"};

static void data_check(u8_t *data, size_t len)
{
    int i,param[6];
    unsigned char validity = true;
    unsigned char length_of_tokens = 0;


    char* curr_token = strtok(data, " ");
    if(curr_token == NULL)
      return;

    int curr_len = strlen(curr_token);

  //  for(i=0; i<(sizeof(known_commands)/sizeof(char*)); i++)
  //  for(i = 0; i < sizeof(known_commands); i++)
    for(i = 0; i < 7; i++)
    {
      if(curr_len == strlen(known_commands[i]) && strncmp(data, known_commands[i], len) == 0)
      {
        switch(i)
        {
          case 0:
                   printk("\n\n\n Idle! \n\n\n\n");
                   data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                          "Message IDLE Received", 21); 
                                      //    message_ok,  sizeof(message_ok)); 
                   new_command = true;
                   break;
          case 1:
                   printk("\n\n\n Stop! \n\n\n\n");
                   data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                          "Message STOP Received", 21); 
                                      //    message_ok,  sizeof(message_ok)); 
                   StopActivity();
                   new_command = true;
                   break;
          case 2:
                   printk("\n\n\n Time \n\n\n\n");
                   length_of_tokens = 0;
          

                   printk("First Token is =  %s\n",curr_token);
                   curr_token = strtok(NULL, " "); 
       
                   while(curr_token != NULL)
                   {                                           
                       param[length_of_tokens] = atoi(curr_token);
                       length_of_tokens++;
                       curr_token = strtok(NULL, " ");   
                       if(length_of_tokens > 3) break;
                   }

                   printk("Length of Tokens =  %d\n",length_of_tokens);


                   // List of Validity checks.
                   if(length_of_tokens < 4) validity = false;
                   if((param[0] < 1) || (param[0] > 7)) validity = false;
                   if(param[1] > 23) validity = false;
                   if(param[2] > 59) validity = false;
                   if(param[3] > 59) validity = false;

                   if(validity)
                   {
                      printk("Time is param0 = %d , param1 = %d  , param2 = %d  , param3= %d \n", param[0], param[1] , param[2], param[3]);                    
                   //   const char *message_ok = "Message is OK";      
                       data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                          "Message TIME Received", 21); 
                                      //    message_ok,  sizeof(message_ok));


                      _CURRENT_DATE_TIME.days =  param[0];
                      _CURRENT_DATE_TIME.hours  =  param[1];
                      _CURRENT_DATE_TIME.minutes  =  param[2];
                      _CURRENT_DATE_TIME.seconds  =  param[3];
                       new_command = true;                   
                   }
                   else
                   {
                     // Echo back Error message.
                     // const char *error_message = "Errror In Message";               
                      data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                  //      error_message, sizeof(error_message)); 
                                      "Error In Message",16);            
                   }                                           
                   break;
           case 3:
                   printk("\n\n\n Preset \n\n\n\n");

                   length_of_tokens = 0;
          

                   printk("First Token is =  %s\n",curr_token);
                   curr_token = strtok(NULL, " "); 
       
                   while(curr_token != NULL)
                   {                                           
                       param[length_of_tokens] = atoi(curr_token);
                       length_of_tokens++;
                       curr_token = strtok(NULL, " ");   
                       if(length_of_tokens > 5) break;
                   }

                   printk("Length of Tokens =  %d\n",length_of_tokens);


                   // List of Validity checks.
                   if(length_of_tokens < 4) validity = false;
                   if(param[0] > 20) validity = false;            
                   if(param[2] == 0) validity = false;
               //    if(param[3] > 24) validity = false;
                   if(param[4] > 24) validity = false;

                   if(validity)
                   {
                        printk("Preset param0 = %d , param1 = %d  , param2 = %d  , param3= %d , param3= %d\n",
                                                     param[0], param[1] , param[2], param[3] , param[4]);                    
                     //   const char *message_ok = "Message is OK";      
                         data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                            "Message PRESET Received", 23); 
                                        //    message_ok,  sizeof(message_ok)); 
                           
                         new_command = true; 
                   }
                   else
                   {
                       // Echo back Error message.
                       // const char *error_message = "Errror In Message";               
                        data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                    //      error_message, sizeof(error_message)); 
                                        "Error In Message",16);    
                                                
                        _AFE4900_RECIPE.test_preset = afe4900_test_preset;
                        _AFE4900_RECIPE.test_type = param[0];
                        // Extract unsigned int to day,hour,minute,second.                       
                        _AFE4900_RECIPE.start_time = param[1];
                        _AFE4900_RECIPE.recipe_duration = param[2];
                        _AFE4900_RECIPE.recipe_interval = param[3];
                        _AFE4900_RECIPE.recipes_num = param[4];

                        // Start Aquire mode
                        // _AFE4900_AQUIRE_MODE.aquire_mode = afe4900_aquire_sample;
                   }                                         
                   break;
                   
           case 4:
                   printk("\n\n\n Interval \n\n\n\n");

                   length_of_tokens = 0;
          
                   printk("First Token is =  %s\n",curr_token);
                   curr_token = strtok(NULL, " "); 
       
                   while(curr_token != NULL)
                   {                                           
                         param[length_of_tokens] = atoi(curr_token);
                         length_of_tokens++;
                         curr_token = strtok(NULL, " ");   
                         if(length_of_tokens > 5) break;
                   }

                   printk("Length of Tokens =  %d\n",length_of_tokens);


                   // List of Validity checks.
                   if(length_of_tokens < 5) validity = false;
                   if(param[0] > 20) validity = false;            
                   if(param[2] == 0) validity = false;
             

                   if(validity)
                   {
                        printk("Preset param0 = %d , param1 = %d  , param2 = %d  , param3= %d , param3= %d\n",
                                                     param[0], param[1] , param[2], param[3] , param[4]);                      
                     //   const char *message_ok = "Message is OK";      
                         data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                            "Message INTERVAL Received", 25); 
                                        //    message_ok,  sizeof(message_ok));
                         new_command = true;                 
                   }
                   else
                   {
                       // Echo back Error message.
                       // const char *error_message = "Errror In Message";               
                        data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                    //      error_message, sizeof(error_message)); 
                                        "Error In Message",16);            
                   }                                          
                   break;               
            case 5:
                   printk("\n\n\n Continuous \n\n\n\n");

                   length_of_tokens = 0;
          

                   printk("First Token is =  %s\n",curr_token);
                   curr_token = strtok(NULL, " "); 
       
                   while(curr_token != NULL)
                   {                                           
                      param[length_of_tokens] = atoi(curr_token);
                      length_of_tokens++;
                      curr_token = strtok(NULL, " ");   
                      if(length_of_tokens > 3) break;
                   }

                   printk("Length of Tokens =  %d\n",length_of_tokens);


                   // List of Validity checks.
                   if(length_of_tokens < 3) validity = false;
                   if(param[0] > 20) validity = false;            
                   if(param[2] == 0) validity = false;           

                   if(validity)
                   {
                      printk("Continuous param0 = %d , param1 = %d  , param2 = %d  \n", param[0], param[1] , param[2]);                    
                   //   const char *message_ok = "Message is OK";      
                       data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                          "Message CONT Received", 21); 
                                      //    message_ok,  sizeof(message_ok)); 
                       new_command = true;   
                   }
                   else
                   {
                     // Echo back Error message.
                     // const char *error_message = "Errror In Message";               
                      data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                  //      error_message, sizeof(error_message)); 
                                      "Error In Message",16);            
                   }                                    
                   break;  
          case 6:
                   printk("\n\n\n Sleep \n\n\n\n");

                   length_of_tokens = 0;
          

                   printk("First Token is =  %s\n",curr_token);
                   curr_token = strtok(NULL, " "); 
       
                   while(curr_token != NULL)
                   {                                           
                      param[length_of_tokens] = atoi(curr_token);
                      length_of_tokens++;
                      curr_token = strtok(NULL, " ");   
                      if(length_of_tokens > 1) break;
                   }

                   printk("Length of Tokens =  %d\n",length_of_tokens);


                   // List of Validity checks.
                   if(param[0] == 0) validity = false;            
                      

                   if(validity)
                   {
                      printk("Sleep param0 = %d  \n", param[0] );                    
                   //   const char *message_ok = "Message is OK";      
                       data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                          "Message SLEEP Received", 22); 
                                      //    message_ok,  sizeof(message_ok)); 
                       new_command = true;   
                   }
                   else
                   {
                     // Echo back Error message.
                     // const char *error_message = "Errror In Message";               
                      data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
                                  //      error_message, sizeof(error_message)); 
                                      "Error In Message",16);            
                   }                                    
                   break; 
          default:
                  printk("DEFAULT WILL NEVER BE CALLED!");
                  break;
        }
        break;
      }
    }


    data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,"Just for test \n", 15);
  //  if(i==(sizeof(known_commands)/sizeof(char*)))
//    if(i==sizeof(known_commands))
//    {
//      data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
//				"Error no command found  !!!", 27); // payload_buf, p->message.payload.len
//    }
}

/**@brief Function to publish data on the configured topic
 */
static int data_publish(struct mqtt_client *c, enum mqtt_qos qos,
	u8_t *data, size_t len)
{
	struct mqtt_publish_param param;

	param.message.topic.qos = qos;
	param.message.topic.topic.utf8 = CONFIG_MQTT_PUB_TOPIC;
	param.message.topic.topic.size = strlen(CONFIG_MQTT_PUB_TOPIC);
	param.message.payload.data = data;
	param.message.payload.len = len;
	param.message_id = sys_rand32_get();
	param.dup_flag = 0;
	param.retain_flag = 0;

	data_print("Publishing: ", data, len);
        // Ephraim 
//	printk("to topic: %s len: %u\n",
//		CONFIG_MQTT_PUB_TOPIC,
//		(unsigned int)strlen(CONFIG_MQTT_PUB_TOPIC));

	return mqtt_publish(c, &param);
}

/**@brief Function to subscribe to the configured topic
 */
static int subscribe(void)
{
	struct mqtt_topic subscribe_topic = {
		.topic = {
			.utf8 = CONFIG_MQTT_SUB_TOPIC,
			.size = strlen(CONFIG_MQTT_SUB_TOPIC)
		},
		.qos = MQTT_QOS_1_AT_LEAST_ONCE
	};

	const struct mqtt_subscription_list subscription_list = {
		.list = &subscribe_topic,
		.list_count = 1,
		.message_id = 1234
	};

	printk("Subscribing to: %s len %u\n", CONFIG_MQTT_SUB_TOPIC,
		(unsigned int)strlen(CONFIG_MQTT_SUB_TOPIC));

	return mqtt_subscribe(&client, &subscription_list);
}

/**@brief Function to read the published payload.
 */
static int publish_get_payload(struct mqtt_client *c, size_t length)
{
	u8_t *buf = payload_buf;
	u8_t *end = buf + length;

	if (length > sizeof(payload_buf)) {
		return -EMSGSIZE;
	}

	while (buf < end) {
		int ret = mqtt_read_publish_payload(c, buf, end - buf);

		if (ret < 0) {
			int err;

			if (ret != -EAGAIN) {
				return ret;
			}

			printk("mqtt_read_publish_payload: EAGAIN\n");

			err = poll(&fds, 1, K_SECONDS(CONFIG_MQTT_KEEPALIVE));
			if (err > 0 && (fds.revents & POLLIN) == POLLIN) {
				continue;
			} else {
				return -EIO;
			}
		}

		if (ret == 0) {
			return -EIO;
		}

		buf += ret;
	}

	return 0;
}

/**@brief MQTT client event handler
 */
void mqtt_evt_handler(struct mqtt_client *const c,
		      const struct mqtt_evt *evt)
{
	int err;

	switch (evt->type) {
	case MQTT_EVT_CONNACK:
		if (evt->result != 0) {
			printk("MQTT connect failed %d\n", evt->result);
			break;
		}

		connected = true;
		printk("[%s:%d] MQTT client connected!\n", __func__, __LINE__);
		subscribe();
		break;

	case MQTT_EVT_DISCONNECT:
		printk("[%s:%d] MQTT client disconnected %d\n", __func__,
		       __LINE__, evt->result);

		connected = false;
		break;

	case MQTT_EVT_PUBLISH: {
		const struct mqtt_publish_param *p = &evt->param.publish;

		printk("[%s:%d] MQTT PUBLISH result=%d len=%d\n", __func__,
		       __LINE__, evt->result, p->message.payload.len);
		err = publish_get_payload(c, p->message.payload.len);
		if (err >= 0) {
			data_print("Received: ", payload_buf,
				p->message.payload.len);
                        data_check(payload_buf,	p->message.payload.len);
                       	/* Echo back received data */
			//data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE,
			//	payload_buf, p->message.payload.len);
		} else {
			printk("mqtt_read_publish_payload: Failed! %d\n", err);
			printk("Disconnecting MQTT client...\n");

			err = mqtt_disconnect(c);
			if (err) {
				printk("Could not disconnect: %d\n", err);
			}
		}
	} break;

	case MQTT_EVT_PUBACK:
		if (evt->result != 0) {
                         // Ephraim
			//printk("MQTT PUBACK error %d\n", evt->result);
			break;
		}

                // Ephraim
	//	printk("[%s:%d] PUBACK packet id: %u\n", __func__, __LINE__,
	//			evt->param.puback.message_id);
		break;

	case MQTT_EVT_SUBACK:
		if (evt->result != 0) {
			printk("MQTT SUBACK error %d\n", evt->result);
			break;
		}

		printk("[%s:%d] SUBACK packet id: %u\n", __func__, __LINE__,
				evt->param.suback.message_id);
		break;

	default:
		printk("[%s:%d] default: %d\n", __func__, __LINE__,
				evt->type);
		break;
	}
}

/**@brief Resolves the configured hostname and
 * initializes the MQTT broker structure
 */
static void broker_init(void)
{
	int err;
	struct addrinfo *result;
	struct addrinfo *addr;
	struct addrinfo hints = {
		.ai_family = AF_INET,
		.ai_socktype = SOCK_STREAM
	};

	err = getaddrinfo(CONFIG_MQTT_BROKER_HOSTNAME, NULL, &hints, &result);
	if (err) {
		printk("ERROR: getaddrinfo failed %d\n", err);

		return;
	}

	addr = result;
	err = -ENOENT;

	/* Look for address of the broker. */
	while (addr != NULL) {
		/* IPv4 Address. */
		if (addr->ai_addrlen == sizeof(struct sockaddr_in)) {
			struct sockaddr_in *broker4 =
				((struct sockaddr_in *)&broker);
			char ipv4_addr[NET_IPV4_ADDR_LEN];

			broker4->sin_addr.s_addr =
				((struct sockaddr_in *)addr->ai_addr)
				->sin_addr.s_addr;
			broker4->sin_family = AF_INET;
			broker4->sin_port = htons(CONFIG_MQTT_BROKER_PORT);

			inet_ntop(AF_INET, &broker4->sin_addr.s_addr,
				  ipv4_addr, sizeof(ipv4_addr));
			printk("IPv4 Address found %s\n", ipv4_addr);

			break;
		} else {
			printk("ai_addrlen = %u should be %u or %u\n",
				(unsigned int)addr->ai_addrlen,
				(unsigned int)sizeof(struct sockaddr_in),
				(unsigned int)sizeof(struct sockaddr_in6));
		}

		addr = addr->ai_next;
		break;
	}

	/* Free the address. */
	freeaddrinfo(result);
}

/**@brief Initialize the MQTT client structure
 */
static void client_init(struct mqtt_client *client)
{
	mqtt_client_init(client);

	broker_init();

	/* MQTT client configuration */
	client->broker = &broker;
	client->evt_cb = mqtt_evt_handler;
	client->client_id.utf8 = (u8_t *)CONFIG_MQTT_CLIENT_ID;
	client->client_id.size = strlen(CONFIG_MQTT_CLIENT_ID);
	client->password = NULL;
	client->user_name = NULL;
	client->protocol_version = MQTT_VERSION_3_1_1;

	/* MQTT buffers configuration */
	client->rx_buf = rx_buffer;
	client->rx_buf_size = sizeof(rx_buffer);
	client->tx_buf = tx_buffer;
	client->tx_buf_size = sizeof(tx_buffer);

	/* MQTT transport configuration */
	client->transport.type = MQTT_TRANSPORT_NON_SECURE;
}

/**@brief Initialize the file descriptor structure used by poll.
 */
static int fds_init(struct mqtt_client *c)
{
	if (c->transport.type == MQTT_TRANSPORT_NON_SECURE) {
		fds.fd = c->transport.tcp.sock;
	} else {
#if defined(CONFIG_MQTT_LIB_TLS)
		fds.fd = c->transport.tls.sock;
#else
		return -ENOTSUP;
#endif
	}

	fds.events = POLLIN;

	return 0;
}

/**@brief Configures modem to provide LTE link. Blocks until link is
 * successfully established.
 */
static void modem_configure(void)
{
#if defined(CONFIG_LTE_LINK_CONTROL)
	if (IS_ENABLED(CONFIG_LTE_AUTO_INIT_AND_CONNECT)) {
		/* Do nothing, modem is already turned on
		 * and connected.
		 */
	} else {
#if defined(CONFIG_LWM2M_CARRIER)
		/* Wait for the LWM2M_CARRIER to configure the modem and
		 * start the connection.
		 */
		printk("Waitng for carrier registration...\n");
		k_sem_take(&carrier_registered, K_FOREVER);
		printk("Registered!\n");
#else /* defined(CONFIG_LWM2M_CARRIER) */
		int err;

		printk("LTE Link Connecting ...\n");
		err = lte_lc_init_and_connect();
		__ASSERT(err == 0, "LTE link could not be established.");
		printk("LTE Link Connected!\n");
#endif /* defined(CONFIG_LWM2M_CARRIER) */
	}
#endif /* defined(CONFIG_LTE_LINK_CONTROL) */
}

void mqtt_publish_msg(u8_t *data, size_t len)
{
        for(;;)
        {
                data_publish(&client, MQTT_QOS_1_AT_LEAST_ONCE, data, len);
                printk("\tpublish done\n");
                k_sleep(1000);
        }
}


void ExtractTimeDate(unsigned int i)
{
    _CURRENT_DATE_TIME.days = (unsigned char) (i / 86400);
    _CURRENT_DATE_TIME.hours = (unsigned char) ((i / 3600) % 24);
    _CURRENT_DATE_TIME.minutes = (unsigned char) ((i / 60) % 60);
    _CURRENT_DATE_TIME.seconds = (unsigned char) (i % 60);
}



unsigned int TimeDate(void)
{
    unsigned int a;

    if(++_CURRENT_DATE_TIME.seconds > 59)
    {
      _CURRENT_DATE_TIME.seconds = 0;
      if(++_CURRENT_DATE_TIME.minutes > 59)
      {
        _CURRENT_DATE_TIME.minutes = 0;
        if(++_CURRENT_DATE_TIME.hours > 23)
        {
          _CURRENT_DATE_TIME.hours = 0;
          if(++_CURRENT_DATE_TIME.days > 7)
           _CURRENT_DATE_TIME.days = 1;
        }
      }
    }

    a = (unsigned int) _CURRENT_DATE_TIME.seconds +
        (unsigned int) (_CURRENT_DATE_TIME.minutes * 60) +
        (unsigned int) (_CURRENT_DATE_TIME.hours * 3600) + 
        (unsigned int) ((_CURRENT_DATE_TIME.days - 1) * 86400) ;
    return a;
}




void my_expiry_function(struct k_timer *timer_id) 
{
  //      ++toggle;
  //      gpio_pin_write(dev,21, toggle % 2);

//        printk("dDay %d Hour %d Minute %d Second %d \r\n",_CURRENT_DATE_TIME.days,
//                                                          _CURRENT_DATE_TIME.hours,
//                                                          _CURRENT_DATE_TIME.minutes,
//                                                          _CURRENT_DATE_TIME.seconds);
        gtimer = TimeDate();

        
        #ifdef NO_MQTT
            switch(_MQTT_SLEEP_STATUS.mqtt_sleep_state)
            {
                case MQTT_sleep_idle :  printk("MQTT sleep status -> sleep_off \n"); 
                                        _MQTT_SLEEP_STATUS.mqtt_sleep_state = MQTT_sleep_off;
                                        connect_off = 0;
                                        connect_on = 0;
                                        break;
                case MQTT_sleep_off :   if(++connect_off >= _MQTT_SLEEP_STATUS.mqtt_sleep_time)
                                        {
                                           printk("MQTT sleep status -> sleep_on \n"); 
                                           _MQTT_SLEEP_STATUS.mqtt_sleep_state = MQTT_sleep_on;
                                           connect_off = 0;
                                        }
                                        break;
                case MQTT_sleep_on :    if(++connect_on >= _MQTT_SLEEP_STATUS.mqtt_connect_time)
                                        {
                                           printk("MQTT sleep status -> sleep_on \n"); 
                                           _MQTT_SLEEP_STATUS.mqtt_sleep_state = MQTT_sleep_idle;
                                           connect_on = 0;
                                        }
                                        break;        
            }
        #endif

        es008c();
}




void initStructures(void)
{
    // Init MQTT Structure.
    _MQTT_SM_STATUS.mqtt_sm_status           =   MQTT_idle;
    _MQTT_CONNECT_STATUS.mqtt_connect_status =   MQTT_connect_idle;
    _MQTT_SLEEP_STATUS.mqtt_sleep_state      =   MQTT_sleep_idle;
    _MQTT_SLEEP_STATUS.mqtt_sleep_time       =   30;   // Change later to 60 minutes
    _MQTT_SLEEP_STATUS.mqtt_connect_time     =   10;   // Seconds.
    _MQTT_ACCEL_STATUS.mqtt_accel_status     =   MQTT_ACCEL_idle;


    // Init CURRENT_DATE_TIME Structure. 
    _CURRENT_DATE_TIME.seconds = 0;
    _CURRENT_DATE_TIME.minutes = 0;
    _CURRENT_DATE_TIME.hours = 8;
    _CURRENT_DATE_TIME.days = 1;

    // Init _AFE4900_OPERATE Structure.
    _AFE4900_OPERATE.afe4900_operate_status = idle;
    _AFE4900_SM.afe4900_test_type = none;

    // ESP008 Operate Structure.
    _ESP008_OPERATE.esp008_operate_status = esp_idle;

    // _AFE4900_RECIPE Structure.
    // Vraiables for Test_Prest Mode.
    _AFE4900_RECIPE.test_preset = afe4900_test_preset; 
//    _AFE4900_RECIPE.test_type = none;
//    _AFE4900_RECIPE.recipe_duration = 8;
//    _AFE4900_RECIPE.recipes_num = 2; 
//    future_time = 19 +        // Seconds.
//                   0 * 60 +    // Minutes.
//                   8 * 3600 +  // Hour.
//                   0;          // Dyas.
//
//     _AFE4900_RECIPE.start_time =  future_time;                                         
//    _AFE4900_RECIPE.stop_time = future_time + 201;
//    _AFE4900_RECIPE.recipe_interval = 5;


    // Vraiables for Test_Interval Mode.
    _AFE4900_RECIPE.test_preset = afe4900_test_interval;
    _AFE4900_RECIPE.recipe_duration = 13;
    _AFE4900_RECIPE.recipes_num = 5; 
    future_time = 19 +        // Seconds.
                   0 * 60 +    // Minutes.
                   8 * 3600 +  // Hour.
                   0 * 86400;  // Days.
    //  m ax Stop time 12 hours. 
    _AFE4900_RECIPE.stop_time = 12 * 3600;      


    // Vraiables for Test_Continuous Mode.
    _AFE4900_RECIPE.test_preset = afe4900_test_cont;
    _AFE4900_RECIPE.recipe_duration = 400;
 //   _AFE4900_RECIPE.recipes_num = 5; 
    future_time = 19 +        // Seconds.
                   0 * 60 +    // Minutes.
                   8 * 3600 +  // Hour.
                   0 * 86400;  // Days.
    //  m ax Stop time 12 hours. 
//    _AFE4900_RECIPE.stop_time = 12 * 3600;       
//     _AFE4900_RECIPE.start_time =  future_time;   

    
 //   _AFE4900_RECIPE.recipe_interval = 5;



    _AFE4900_RECIPE.save = true;
    _AFE4900_RECIPE.wire = true;

}

void TestTimeExtract(void)
{
    unsigned int k;

    _CURRENT_DATE_TIME.seconds = 29;
    _CURRENT_DATE_TIME.minutes = 59;
    _CURRENT_DATE_TIME.hours = 6;
    _CURRENT_DATE_TIME.days = 2;

    k = TimeDate();
    printk("k =  %d   \n",k);

    ExtractTimeDate(k);


    printk("day %d hour %d minute %d second %d \n", _CURRENT_DATE_TIME.days ,
                                                    _CURRENT_DATE_TIME.hours ,
                                                    _CURRENT_DATE_TIME.minutes ,
                                                    _CURRENT_DATE_TIME.seconds);
 // Ephraim ?
 //  while(1) {} ;
}

unsigned char Wait4Command(void)
{
    if(new_command == true)
    {
       new_command = false;
       return true;
    }
    else
       return false;
}


void RetriveData(void)
{
    // Get test mode afe4900_test_preset,afe4900_test_interval,afe4900_test_cont
    _AFE4900_RECIPE.test_preset = afe4900_test_preset;
    switch(_AFE4900_RECIPE.test_preset)
    {
        case afe4900_test_idle  :

                                  break;
        case afe4900_test_preset:  
                                  break;
        case afe4900_test_interval:
                                  break;
        case afe4900_test_cont:
                                  break;                            
    }
      
 //     AFE4900_SM.afe4900_test_type = green;
}


void EstimateFlashSize(void)
{
    // Assume max data size 10 bytes for each test_type.
    switch(_AFE4900_RECIPE.test_preset)
    {
        case afe4900_test_idle  :

                                  break;
        case afe4900_test_preset: _MT25QI128_MEMORY.mt_estimated_chunk = (unsigned int)
                                             (200 * _AFE4900_RECIPE.recipe_duration
                                            * _AFE4900_RECIPE.recipes_num); 
                                  break;
        case afe4900_test_interval:
                                  break;
        case afe4900_test_cont:
                                  break;                            
    }
    
}


void UpdateMT25QL128_data(void)
{
    _MT25QI128_MEMORY.mt_current_pointer = 0x0;    // Current pointer of address.
    _MT25QI128_MEMORY.mt_start   = 0x0;            // Start of SPI flash's address.
    _MT25QI128_MEMORY.mt_max_length   = 0x3ff;     // Max bytes to write.
    // mt_chunk to be updated by test type.
    // mt_last_pointer incremented after each session.
    // mt_buffer[] update after each session.
}


unsigned char Wait4StartTime(unsigned int timestart)
{
   PrintTimer(gtimer);
   PrintTimer(timestart);

   if(gtimer >= timestart)
     return true;
   else
     return false;
}


void PrintTimer(unsigned int timer)
{
    unsigned char m_seconds;
    unsigned char m_minutes;
    unsigned char m_hours;
    unsigned char m_days;

    m_seconds = (unsigned char) (timer % 60);
    m_minutes = (unsigned char) ((timer / 60) % 60);
    m_hours   = (unsigned char) ((timer / 3600) % 24);
    m_days    = (unsigned char) (timer / (3600 * 24));

    printk("timer days hours minutes seconds %d %d %d %d %d !\r\n",timer,m_days,m_hours,m_minutes,m_seconds);
}


unsigned char wire_data(void)
{

   return true;
}



void es008c(void)
{
   static char m_numTest = 0;
   static int m_duration = 0;
   static unsigned int m_interval;
   static unsigned int start_of_interval;

//     gtimer = TimeDate();
   // Ephraim.
//   return;

     switch(_ESP008_OPERATE.esp008_operate_status)
     {
        case esp_idle: // Wait 4 new command , Open MQTT every 30 seconds.
                      // _ESP008_OPERATE.esp008_operate_status = esp_wait4command;                     
                       m_numTest = 0;
               ///        printk("esp_idle \r\n");                   
                       break;
        case esp_wait4command: // Wait 4 time2 start.
                           //    #ifdef NO_MQTT
                               if(Wait4Command() == true)
                               {
                                 RetriveData();  // Include StartTime,Duration,Num od tets,Type of test.
                                 // Check Validity of data , Later
                                 EstimateFlashSize();
                                 _ESP008_OPERATE.esp008_operate_status = esp_in_process; 
                               }
                                   // Here update data for spi flash and afe4900.
                                   UpdateMT25QL128_data();
                                   start_of_interval = gtimer; 
                          //     #endif
                          //      printk("esp_pre_process \r\n");
                          //     _ESP008_OPERATE.esp008_operate_status = esp_pre_process;                             
                       break;
        case esp_pre_process:
                         switch (_AFE4900_RECIPE.test_preset)
                         {
                      //        case afe4900_test_idle  : 
                                                        //_ESP008_OPERATE.esp008_operate_status = afe4900_test_idle;
                                                      
                      //                                  break;
                              case afe4900_test_preset:  
                                                   
                                                      //    #ifdef NO_MQTT
                                                                if(Wait4StartTime(_AFE4900_RECIPE.start_time) == true)
                                                                 {
                                                                    _ESP008_OPERATE.esp008_operate_status = esp_in_process; 
                                                                     m_duration = 0;                                                         
                                                     

                                                                    PrintTimer(gtimer);
                                                                    printk("Start m_numTest %d \r\n",m_numTest);
                                                                    printk("Go to esp_in_process %d \r\n",m_numTest);
                                                                    
                                                                 }
                                                     //      #endif
                                                     //      printk("esp_in_process %d \r\n",m_numTest);
                                                         break;
                              case afe4900_test_interval:
                              
                                                         _AFE4900_RECIPE.start_time = gtimer;
                                                          printk("gtimer is sets %d \r\n",gtimer);
                                                          m_duration = 0; 
                                                         _ESP008_OPERATE.esp008_operate_status = esp_in_process; 

                                                         if(gtimer > (start_of_interval + _AFE4900_RECIPE.stop_time))
                                                         {
                                                             _ESP008_OPERATE.esp008_operate_status = esp_idle;
                                                             printk("end of process \r\n"); 
                                                                                               }
                                                         break;
                              case afe4900_test_cont:    
                                                          _AFE4900_RECIPE.start_time = gtimer;
                                                          _AFE4900_RECIPE.stop_time += gtimer;
                                                          printk("gtimer is sets %d \r\n",gtimer);
                                                          printk("end time  is %d \r\n", _AFE4900_RECIPE.stop_time);
                                                          m_duration = 0;  
                                                         _ESP008_OPERATE.esp008_operate_status = esp_in_process; 
                                                         break; 
                         }
                       break;
        case esp_in_process:
                        switch (_AFE4900_RECIPE.test_preset)
                         {
                         //     case afe4900_test_idle  :

                         //                               break;
                              case afe4900_test_preset:  if(++m_duration > _AFE4900_RECIPE.recipe_duration)
                                                         {                                                                                                                      
                                                            if(m_numTest >= (_AFE4900_RECIPE.recipes_num - 1))
                                                            {
                                                              _ESP008_OPERATE.esp008_operate_status = esp_end;// esp_idle; change later                                                              
                                                            }
                                                            else
                                                            {
                                                      //          printk("m_numTest %d \r\n",m_numTest);
                                                                printk("End of m_numTest %d \r\n",m_numTest);
                                                                printk("Go to Interval %d \r\n");
                                                      //         _ESP008_OPERATE.esp008_operate_status = esp_pre_process;
                                                                 _ESP008_OPERATE.esp008_operate_status = esp_interval;
                                                            }
                                                             m_interval = 0;
                                                             m_numTest++;
                                                         }    
                                                         break;
                              case afe4900_test_interval:if(++m_duration >= _AFE4900_RECIPE.recipe_duration)
                                                         {                                                          
                                                              PrintTimer(gtimer);
                                                              printk("Start of interval  \r\n");
                                                              m_interval = 0;
                                                              _ESP008_OPERATE.esp008_operate_status = esp_interval;                                                      
                                                         }
                                                       
                                                         break;
                              case afe4900_test_cont:    if(++m_duration >=  _AFE4900_RECIPE.recipe_duration)
                                                         {    
                                                              m_interval = 0;
                                                              m_duration = 0;

                                                              if(gtimer >= _AFE4900_RECIPE.stop_time)
                                                              {
                                                                 _ESP008_OPERATE.esp008_operate_status = esp_wire_data_abort;  // change later                                                            
                                                              }
                                                              else
                                                              {
                                                                 _ESP008_OPERATE.esp008_operate_status = esp_wire_data;                                                                 
                                                              }
                                                         }
                                                         printk("gtimer,_AFE4900_RECIPE.recipe_duration,m_duration %d %d %d \r\n",gtimer,_AFE4900_RECIPE.recipe_duration,m_duration);                                                                                      
                                                         break; 
                         }
                       break;
        case esp_interval: if(++m_interval >= _AFE4900_RECIPE.recipe_interval)
                       {
                          // Check for new command.
                          _ESP008_OPERATE.esp008_operate_status = esp_pre_process;
                          printk("End of interval  \r\n");
                          PrintTimer(gtimer); 
                       }
                       break;
        case esp_end:  // Clear all data , later.
                       _ESP008_OPERATE.esp008_operate_status = esp_idle;//esp_pre_process;
                       printk("end of process \r\n");
                       break;
        case esp_wire_data:
                       // Wire data.
                       wire_data();
                       PrintTimer(gtimer);
                       printk("esp_wire_data \r\n");
                       _ESP008_OPERATE.esp008_operate_status = esp_in_process;
                       break;
        case esp_wire_data_abort: 
                       // Wire data.
                       wire_data();
                       PrintTimer(gtimer);
                       printk("esp_wire_data_abort \r\n");
                       _ESP008_OPERATE.esp008_operate_status = esp_abort;
                       break;
        case esp_abort:
                       // Clear   all data                  
                       printk("we are done \r\n");
                       break;
     }

}

 

void AFE4900_Interrupt(struct device *gpiob, struct gpio_callback *cb,u32_t pins)
{
    unsigned char a,b;
     //  printk("Button pressed at %" PRIu32 "\n", k_cycle_get_32());
     // next line for debugging.
  //   static unsigned int period = 0;

//     SetLed(RED,ON);
//     SetLed(BLUE,OFF);
//     while(1) {};
//     return;
     switch(_AFE4900_RECIPE.test_type)
     {
        case none:    if(_AFE4900_AQUIRE_MODE.aquire_mode ==  afe4900_aquire_sample)
                      {      
                          SPI_read_continuous(0x2a,2);
                          flash_page_counter += afe4900_mem_chunk;
                        
                     //     for(a = 0;a < 120;a++)
                  //          b = a + 1;
           //              MT25qlxx_Reset();
            //             MT25qlxx_ReadUniqID();
                       MT25qlxx_WriteEnable();

                          // Wait for Write to end , 
                          if(afe4900_int_counter == 30)
                          { 
               //             MT25qlxx_WriteEnable();
                              Wait4EraeWrite();                 
                          }

                   
                       
                           if(++afe4900_int_counter >= 40)
                           {  
                    //            ReadAccelerometerXYZ();
                                flash_buffer[flash_bank][240] = 0x01;
                                flash_buffer[flash_bank][241] = 0x02;
                                flash_buffer[flash_bank][242] = 0x03;
                                flash_buffer[flash_bank][243] = 0x04;
                                flash_buffer[flash_bank][244] = 0x05;
                                flash_buffer[flash_bank][245] = 0x06;
                                flash_page_counter += lis2dh12_mem_chunk;  
                          //    adc_sample(); 
                                flash_buffer[flash_bank][246] = 0x22;
                                flash_buffer[flash_bank][247] = 0x12;
                                flash_page_counter += adc_mem_chunk;
                          //    Add Etrog_counter;
                                EtrogCounter();
                                flash_page_counter += etrog_mem_chunk;  
                       //         MT25qlxx_WriteEnable();
                                // Ephraim Check later.
                                
                      //          if((flash_bank > 1) || (flash_section_counter > 40) || (flash_chunk_counter > 13))
                      //            SetLed(GREEN,ON);
                    //            MT25qlxx_WriteFullPage(&flash_buffer[flash_bank][flash_section_counter * 0x100],0);
                                
                                MT25qlxx_WriteFullPage(&spi_tx_buffer[flash_section_counter * 0x100],0);
                ///             MT25qlxx_ReadFullPage(&flash_buffer[0][0],0);
                                flash_section_counter++;
                            //    flash_page_counter += afe4900_mem_chunk;
                       

                         
      //                    //      MT25qlxx_WriteEnable();
      //                   //       T25qlxx_WritePage(&spi_tx_buffer[0],flash_page_counter,256);
                                afe4900_int_counter = 0;
                                flash_page_counter = 0;

                                if(++flash_chunk_counter >= 12)  
                                {
                      //           MT25qlxx_WriteEnable();
                                   if(flash_bank == 0)
                                     flash_bank = 1;
                                   else
                                     flash_bank = 0;
                                   flash_section_counter = 0;
                                   flash_chunk_counter = 0;
                                   flash_block_switch = true;
                                }

                              }
                            }
                         break;
        case green:      SPI_read_continuous(0x2a,2);
                         break;
        case red:        SPI_read_continuous(0x2a,2);
                         break;
        case ir:         SPI_read_continuous(0x2a,2);
                         break;
        case double_grren: 

                         SPI_read_continuous(0x2a,2);
                         break;
        case green_red: 
                         SPI_read_continuous(0x2a,2);
                         break;
        case green_ir:   SPI_read_continuous(0x2a,2);
                         break;
        case ir_red:     SPI_read_continuous(0x2a,2);
                         break;
     }
}


void AFE4900_Interrupt_Config(void)
{
    gpio_pin_configure(dev_io, GPIO_INT_PIN, GPIO_DIR_IN | GPIO_INT | GPIO_PUD_PULL_UP | GPIO_INT_EDGE);
    gpio_init_callback(&gpio_cb, AFE4900_Interrupt, BIT(GPIO_INT_PIN));
    gpio_add_callback(dev_io, &gpio_cb);
    gpio_pin_enable_callback(dev_io, GPIO_INT_PIN);
}


unsigned char ADP5360_3v3_setup(void)
{
    unsigned char a = true;
//    adp536x_buckbst_3v3_set();
    if(adp536x_reg_write(0x2c, 0x13) != 0)        
    {
       printk("Error on i2c_writed \r\n");
       a = false;
    }

    // Ephraim Change later.
    if(adp536x_reg_write(0x29, 0x31) != 0)        
    {
       printk("Error on i2c_writed \r\n");
       a = false;
    }
    return a;
}


void EnableOscillator(unsigned char onoff)
{
    if(onoff == true)
        gpio_pin_write(dev_io, OSC_ENA , true);
    else
        gpio_pin_write(dev_io, OSC_ENA , false);
}

void EnablePPGPower(unsigned char onoff)
{
    if(onoff == true)
        gpio_pin_write(dev_io, POW_ENA , true);
    else
        gpio_pin_write(dev_io, POW_ENA , false);
}


void SetLed(unsigned char color,unsigned char state)
{
   switch(color)
   {
     case RED:
              switch(state)
              {
                 case ON: 
                          gpio_pin_write(dev_io, RLED , true);
                          break;
                 case OFF:
                          gpio_pin_write(dev_io, RLED , false);
                          break;
              }
              break;
     case BLUE:
              switch(state)
              {
                 case ON: 
                          gpio_pin_write(dev_io, BLED , true);
                          break;
                 case OFF:
                          gpio_pin_write(dev_io, BLED , false);
                          break;
              }   
              break;
     case GREEN:
              switch(state)
              {
                 case ON: 
                          gpio_pin_write(dev_io, GLED , true);
                          break;
                 case OFF:
                          gpio_pin_write(dev_io, GLED , false);
                          break;
              } 
              break;
   } 
}

//const struct spi_config spi_cfg = {
//        .operation = SPI_OP_MODE_MASTER | SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
//		      SPI_MODE_CPOL | SPI_MODE_CPHA ,
//	.frequency = 7000000,
//	.slave = 0,
////        .cs = &spi_cs,
//};

void spi_test_send(void)
{
	int err;
	static u8_t tx_buffer[4] = {0x9f,0x35,0x23,0x55};
	static u8_t rx_buffer[4];

	const struct spi_buf tx_buf = {
		.buf = tx_buffer,
		.len = sizeof(tx_buffer)
	};
	const struct spi_buf_set tx = {
		.buffers = &tx_buf,
		.count = 1
	};

	struct spi_buf rx_buf = {
		.buf = rx_buffer,
		.len = sizeof(rx_buffer),
	};
	const struct spi_buf_set rx = {
		.buffers = &rx_buf,
		.count = 1
	};

//        gpio_pin_write(dev_io, 24, 0);
	err = spi_transceive(spi_dev3, &spi_cfg_afe4900, &tx, &rx);
//         gpio_pin_write(dev_io, 24, 1);
	if (err) {
		printk("SPI error: %d\r\n", err);
                
	} else {
		/* Connect MISO to MOSI for loopback */
//      	        printk("TX sent: %x\n", tx_buffer[0]);
//		printk("RX recv: %x\n", rx_buffer[0]);
	//	tx_buffer[0]++;
	}	
}

//void SPI_write(unsigned char a_iCom,  int a_idata)
//{
//    int err;  
//             
//    static char l_aBuffer[4];
//    static char l_aRxBuffer[4];
//
//  l_aBuffer[0] = a_iCom;
//  l_aBuffer[1] = (char)(a_idata>>16);
//  l_aBuffer[2] = (char)(a_idata>>8);
//  l_aBuffer[3] = (char)(a_idata);
///*
//    const struct spi_buf tx_buf = {
//		.buf = tx_buffer,
//		.len = sizeof(tx_buffer)
//	};
//*/
//    const struct spi_buf_set tx = {
//            .buffers = &l_aBuffer,
//            .count = 4
//    };
///*
//    struct spi_buf rx_buf = {
//            .buf = rx_buffer,
//            .len = sizeof(rx_buffer),
//    };
//*/
//    const struct spi_buf_set rx = {
//            .buffers = &l_aRxBuffer,
//            .count = 4
//    };
//
//
////    struct spi_buf tx_buf = {
////             .buf  = spi_tx_buffer,
////	     .len = 4
////	};
////
////    const struct spi_buf rx_buf = {
////            .buf = spi_rx_buffer,
////            .len = 4
////    };
//
//  
//    
///*
//    tx_buffer[0] = a_iCom;
//    tx_buffer[1] = (unsigned char) ((a_idata >> 16) & 0xff);
//    tx_buffer[2] = (unsigned char) ((a_idata >> 8) & 0xff);
//    tx_buffer[3] = a_idata & 0xff;
//	*/
//    gpio_pin_write(dev_io, 24, 0);
//    err = spi_transceive(spi_dev3, &spi_cfg, &tx, &rx);
//    gpio_pin_write(dev_io, 24, 1);
//    if (err) {
//	printk("SPI error: %d\n", err);
//    }
//}

int adc_sample(void)
{
	int ret;

	const struct adc_sequence sequence = {
		.channels = BIT(ADC_1ST_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) {
		return -1;
	}

	//ret = adc_read(adc_dev, &sequence);
//	printk("ADC read err: %d\n", ret);
        adc_result = &m_sample_buffer[0];


//         memcpy(&flash_buffer[flash_page_counter],&m_sample_buffer[1],adc_mem_chunk);

//	/* Print the AIN0 values */
//	for (int i = 0; i < BUFFER_SIZE; i++) {
//		float adc_voltage = 0;
//		adc_voltage = (float)(((float)m_sample_buffer[i] / 1023.0f) *
//				      3600.0f);
//		printk("ADC raw value: %d\n", m_sample_buffer[i]);
//		printf("Measured voltage: %f mV\n", adc_voltage);
//	}

	return ret;
}


void Flashtest(void)
{
     unsigned char a,b;

     MT25qlxx_Reset();
     MT25qlxx_WriteEnable();
 //    MT25qlxx_EraseChip();
     MT25qlxx_EraseBlock(0); // wait for 123 msec.
     Wait4EraeWrite();
  
 
     MT25qlxx_WriteEnable();
     MT25qlxx_WriteFullPage(&spi_tx_buffer[0],0);
     Wait4EraeWrite();
 
   
       MT25qlxx_ReadFullPage(&spi_rx_buffer[0],0);
       NRFX_DELAY_US(300);
//     MT25qlxx_WriteEnable();
     spi_rx_buffer[255] = 0x25;

//      SPI_read_continuous(0x2a,2);
}


void EtrogCounter(void)
{
    static unsigned int i = 0;
  
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 248] = 'E';
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 249] = 'T';
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 250] = 'R';
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 251] = 'O';
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 252] = 'G';
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 253] = (unsigned char) (i & 0xff);
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 254] = (unsigned char) ((i >> 8) & 0xff);
    flash_buffer[flash_bank][(flash_section_counter * 0x100) + 255] = (unsigned char) ((i >> 16) & 0xff);
 
    i++;
}

void main(void)
{
	int err;
        static unsigned int i = 0;
        static unsigned long periodx = 0;
 
       
        initStructures();
         // Ephraim 26_5_2020
         _AFE4900_AQUIRE_MODE.aquire_mode =  afe4900_aquire_sample;
 
        k_timer_init(&my_timer, my_expiry_function, NULL);
        k_timer_start(&my_timer, K_SECONDS(TIEMR_INTERVAL_SEC), K_SECONDS(TIEMR_INTERVAL_SEC));

        io_init();
    //    SetLed(RED,ON);
    //    SetLed(GREEN,ON);
    //    SetLed(BLUE,ON);
//        gpio_pin_write(dev_io, 0, 1);
//        gpio_pin_write(dev_io, 1, 1);
//        gpio_pin_write(dev_io, 2, 1);
//        EnablePPGPower(true);
//        while(1) {};
//        if(init_i2c2() == false)
//        {
//          SetLed(BLUE,ON);
//          while(1) {};
//        }


//        if(uart_init1() == false) 
//        {
//          SetLed(RED,ON);
//          while(1) {};
//        }
     
        if(adc_init() == false) // ADC.
        {
          SetLed(GREEN,ON);
          while(1) {};
        }
        printk("device_get_binding ADC_0 succedded\n"); 
 

        if(spi_init1() == false) 
        {
          SetLed(BLUE,ON);
          while(1) {};
        }

        if(spi_init3() == false) // AFE4900.
        {
          SetLed(RED,ON);
          while(1) {};
        }

//
//SEGGER_RTT_Init();
//
//
//  do {
//  printk("The MQTT simple sample started\n");
//    SEGGER_RTT_WriteString(1, "Hello World from SEGGER!\n");
//    int i = 100000*100;
//  do { ; } while (i--);
//  } while (1);
//  return 0;

        
//while(1)
//{
//    Flashtest();
//
//}


        
   
//        if(ADP5360_3v3_setup()  == false)
//        {
//           SetLed(RED,ON);
//           while(1) {};
//        }

  //         k_cpu_idle();
  //      }

//        LIS2DH12_Init();
// 
// 
//        MT25qlxx_Reset();
//        MT25qlxx_WriteEnable();
        MT25qlxx_EraseBlock(0);


//        MT25qlxx_EraseChip();
//        Wait4EraeWrite();

        MT25qlxx_WriteEnable();
        MT25qlxx_EraseBlock(1);  
        Wait4EraeWrite();
         
  
       
        AFE4900_RESETZ_Init();
       
        
  //      MT25qlxx_Reset();

        EnablePPGPower(false);
        AFE4900_Reg_Init();
        EnableOscillator(true);
        AFE4900_Interrupt_Config();


//while(1)
//{
//    Flashtest();
//
//}



        while(1)
        {
    //        ReadAccelerometerXYZ();

      //      NRFX_DELAY_US(10000);

 //            SPI_read_continuous(0x2a,2);
 //               SPI_write(0x12,0x34);
           
   //         gpio_pin_write(dev_io, 15, 0);
  //          gpio_pin_write(dev_io, 15, 1);

     //      spi_test_send();
 //          AFE4900_Reg_Write(0x01, 0x000030);
 //             SPI_read_continuous(0x2a,2);
 //             MT25qlxx_WriteEnable();
          

            if(periodx < 200000)
            {
               // Set Colors;
               SetLed(RED,OFF);
               SetLed(BLUE,ON);
            }
            else
            {
               // Set Colors;
               SetLed(RED,ON);
               SetLed(BLUE,OFF);
            }
            if(++periodx > 400000)
              periodx = 0;        
        }

 //    static unsigned long periodx = 0;
//     while(1)
//     {
//        switch(_MQTT_SM_STATUS.mqtt_sm_status)
//        {
//           case  MQTT_idle:  
//                                 #ifdef NO_MQTT
//                                     printk("The MQTT simple sample started\n");
//                                     //SEGGER_RTT_WriteString(0, "The MQTT simple sample started\n");
//                                 #endif
//                                   SetLed(BLUE,ON);
//                                   SetLed(GREEN,ON);
//                                   _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_configure;
//                               break;
//           case  MQTT_configure:
//                                 #ifdef NO_MQTT
//                               //     printk("Configure Modem & Client \n");
//                                     //SEGGER_RTT_printf("Configure Modem & Client \n");
//                                     //SEGGER_RTT_WriteString(0, "The MQ");
//                                     
//                                     modem_configure();
//                                     client_init(&client);
//
//
//                                     err = mqtt_connect(&client);
//                                     if (err != 0) {
//                                            printk("ERROR: mqtt_connect %d\n", err);
//                                             _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_idle;
//                                     }
//
//                                     err = fds_init(&client);
//                                     if (err != 0) {
//                                            printk("ERROR: fds_init %d\n", err);
//                                             _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_idle;
//                                     }
//
//                   //                 SetLed(BLUE,OFF);
//                   //                  SetLed(GREEN,OFF);
//                   //                   SetLed(RED,ON);
//                   //                  while(1) {} ;
//                             
//                                  #endif
//                                    _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll;
//                               break;
////           case  MQTT_connect:
////                                 #ifdef NO_MQTT 
////                                     printk("Configure Connect \n");
////                                     //SEGGER_RTT_printf("Configure Connect \n");
////                                     //SEGGER_RTT_WriteString(0, "The MQ");
////                                     err = mqtt_connect(&client);
////                                     if (err != 0) {
////                                             printk("ERROR: mqtt_connect %d\n", err);
////                                       //      _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_idle;
////                                             SetLed(BLUE,OFF);
////                                             SetLed(GREEN,ON);
////                                             SetLed(BLUE,ON);
////                                        //     SetLed(GREEN,OFF);                         
////                                     }
////                                     SetLed(BLUE,OFF);
////                                     SetLed(GREEN,ON);
////                                 //    SetLed(RED,ON);
////                                 //    while(1) {} ;
////                                 //    SetLed(GREEN,OFF);
////                                      printk("Configure Client \n");
////                                  #endif
////                                 _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll;                             
////                               break;
////           case  MQTT_client:   
////                                  #ifdef NO_MQTT               
////                                     err = fds_init(&client);
////                                     if (err != 0) {
////                                            printk("ERROR: fds_init %d\n", err);
////                                            _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_idle;  
////                                     }   
////                                     SetLed(BLUE,ON);
////                                     SetLed(GREEN,OFF);                                
////                                     printk("MQTT_poll \n");
////                                   #endif
////                                   _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll; 
////                               break;
//           case  MQTT_poll:
//                                 #ifdef NO_MQTT 
//                                     err = poll(&fds, 1, K_SECONDS(CONFIG_MQTT_KEEPALIVE));
//                                     if (err < 0) {
//                                            printk("ERROR: poll %d\n", errno);
//                                             _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll;  
//                                     }
//                                     printk("MQTT_live \n");
//                                 #endif
//                          //       SetLed(BLUE,ON);
//                           //       SetLed(GREEN,ON);   
//                                 _MQTT_SM_STATUS.mqtt_sm_status = MQTT_live;                                       
//                               break;
//           case  MQTT_live:
//                                 #ifdef NO_MQTT 
//                                 err = mqtt_live(&client);
//                                     if (err != 0) {
//                                            printk("ERROR: mqtt_live %d\n", err);
//                                            _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll; 
//                                     }
//                                     printk("MQTT_connected \n");                                 
//                                 #endif
//                                 _MQTT_SM_STATUS.mqtt_sm_status = MQTT_connected;  
//                          //      SetLed(BLUE,ON);
//                                  SetLed(GREEN,ON);
//                          //        SetLed(RED,ON);       
//                               break;
//           case  MQTT_connected:
//                               
//                               // Here Maim loop/
//                               // es008c();
//                               if(periodx < 16)
//                              {
//                                 // Set Colors;
//                                 SetLed(RED,OFF);
//                                 SetLed(BLUE,ON);
//                              }
//                              else
//                              {
//                                 // Set Colors;
//                                 SetLed(RED,ON);
//                                 SetLed(BLUE,OFF);
//                              }
//                              if(++periodx > 32)
//                                periodx = 0;
//
// 
//                               _MQTT_SM_STATUS.mqtt_sm_status = MQTT_revents_pollin; 
//                          //     printk("MQTT_connected \n");
//                               break;
//           case  MQTT_revents_pollin:
//                                  #ifdef NO_MQTT 
//                                     if ((fds.revents & POLLIN) == POLLIN)
//                                     {
//                                         err = mqtt_input(&client);
//                                         if (err != 0) 
//                                         {
//                                                printk("ERROR: mqtt_input %d\n", err);
//                                                _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll; 
//                                         }
//                                         if(new_command == true)
//                                         {
//                                            SetLed(RED,ON);
//                                            SetLed(BLUE,OFF);
//                                            while(1) {} ;
//                                       //    new_command = false;
//                                           printk("new command x \n");
//                                           printk("mqtt_input arrived %d \n",i++);                            
//                                         }  
//                                     }                                                                            
//                                      printk("MQTT_revents_pollerr \n");
//                                   #endif
//                                   _MQTT_SM_STATUS.mqtt_sm_status = MQTT_revents_pollerr;
//                               break;
//           case  MQTT_revents_pollerr:
//                                 #ifdef NO_MQTT 
//                                     if ((fds.revents & POLLERR) == POLLERR)
//                                     {
//                                            printk("POLLERR\n");
//                                            _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll; 
//                                     }                                                 
//                                  #endif
//                                   _MQTT_SM_STATUS.mqtt_sm_status = MQTT_revents_pollnvall;
//                               break;
//           case  MQTT_revents_pollnvall:
//                                #ifdef NO_MQTT 
//                                   if ((fds.revents & POLLNVAL) == POLLNVAL)
//                                   {
//                                        printk("POLLNVAL\n");
//                                        _MQTT_SM_STATUS.mqtt_sm_status =  MQTT_poll; 
//                                   }
//                                 #endif
//                                 _MQTT_SM_STATUS.mqtt_sm_status = MQTT_poll;   
//                               break;                            
//           case  MQTT_disconnected:
//                               #ifdef NO_MQTT 
//                                   printk("Disconnecting MQTT client...\n");
//
//                                   err = mqtt_disconnect(&client);
//                                   if (err)
//                                   {
//                                          printk("Could not disconnect MQTT client. Error: %d\n", err);
//                                   }
//                                #endif
//                                 _MQTT_SM_STATUS.mqtt_sm_status = MQTT_poll;//MQTT_idle; 
//                               break;
//
//      }
//
//     }
       
# if 0
	printk("The MQTT simple sample started\n");

	modem_configure();

	client_init(&client);

	err = mqtt_connect(&client);
	if (err != 0) {
		printk("ERROR: mqtt_connect %d\n", err);
		return;
	}

	err = fds_init(&client);
	if (err != 0) {
		printk("ERROR: fds_init %d\n", err);
		return;
	}

	while (1) {
		err = poll(&fds, 1, K_SECONDS(CONFIG_MQTT_KEEPALIVE));
		if (err < 0) {
			printk("ERROR: poll %d\n", errno);
			break;
		}

		err = mqtt_live(&client);
		if (err != 0) {
			printk("ERROR: mqtt_live %d\n", err);
			break;
		}
                else
                {
                 //   printk("mqtt_connected  \n");
                }

		if ((fds.revents & POLLIN) == POLLIN) {
			err = mqtt_input(&client);
			if (err != 0) {
				printk("ERROR: mqtt_input %d\n", err);
				break;
			}
                        if(new_command == true)
                        {
                           new_command = false;
                           printk("new command x \n");
                           printk("mqtt_input arrived %d \n",i++);
                        }                       
		}

		if ((fds.revents & POLLERR) == POLLERR) {
			printk("POLLERR\n");
			break;
		}

		if ((fds.revents & POLLNVAL) == POLLNVAL) {
			printk("POLLNVAL\n");
			break;
		}
	}

	printk("Disconnecting MQTT client...\n");

	err = mqtt_disconnect(&client);
	if (err) {
		printk("Could not disconnect MQTT client. Error: %d\n", err);
	}

#endif
}
